package de.fhdw.gaming.core.ui.type;

import java.util.Optional;

import de.fhdw.gaming.core.ui.InputProviderException;
import de.fhdw.gaming.core.ui.type.validator.Validator;

/**
 * Allows to perform additional value validation and transformation.
 *
 * @param <T> The type of value.
 */
public final class FieldTypeWithValidator<T> implements FieldType<T> {

    /**
     * The {@link FieldType} to delegate to.
     */
    private final FieldType<T> delegatee;
    /**
     * The {@link Validator} to use.
     */
    private final Validator<T> validator;

    /**
     * Creates an {@link Validator} with a validator.
     *
     * @param delegatee The {@link FieldType} to delegate to.
     * @param validator The {@link Validator} to use.
     */
    public FieldTypeWithValidator(final FieldType<T> delegatee, final Validator<T> validator) {
        this.delegatee = delegatee;
        this.validator = validator;
    }

    /**
     * Returns the delegatee.
     */
    public FieldType<T> getDelegatee() {
        return this.delegatee;
    }

    /**
     * Returns the validator.
     */
    public Validator<T> getValidator() {
        return this.validator;
    }

    /**
     * Adds a validator.
     *
     * @param additionalValidator The {@link Validator} to use.
     * @return The new {@link FieldTypeWithValidator}.
     */
    public FieldTypeWithValidator<T> and(final Validator<T> additionalValidator) {
        return new FieldTypeWithValidator<>(this, additionalValidator);
    }

    @Override
    public void accept(final FieldTypeVisitor visitor) throws FieldTypeException {
        visitor.handle(this);
    }

    @Override
    public Optional<T> getDefaultValue() {
        return this.delegatee.getDefaultValue();
    }

    @Override
    public Optional<String> getInfo() {
        final String info = this.delegatee.getInfo().orElse("");
        return Optional.of(info.isEmpty() ? this.validator.getInfo() : info + "\n" + this.validator.getInfo());
    }

    @Override
    public T postInput(final String id, final String value) throws InputProviderException {
        final T transformedValue = this.delegatee.postInput(id, value);
        return this.validator.validate(transformedValue);
    }
}
