package de.fhdw.gaming.core.ui.util;

import java.util.Map;
import java.util.Optional;
import java.util.Set;

import de.fhdw.gaming.core.domain.util.FunctionE;
import de.fhdw.gaming.core.ui.InputProvider;
import de.fhdw.gaming.core.ui.InputProviderException;
import de.fhdw.gaming.core.ui.type.validator.Validator;

/**
 * An {@link InputProvider} that links two {@link InputProvider}s together.
 */
public final class ChainedInputProvider implements InputProvider {

    /**
     * The current {@link InputProvider} in the chain.
     */
    private final InputProvider current;
    /**
     * A function returning the next {@link InputProvider} in the chain, given the data requested by the current one.
     */
    private final FunctionE<Map<String, Object>, InputProvider, InputProviderException> next;

    /**
     * Creates a {@link ChainedInputProvider}.
     *
     * @param current The current {@link InputProvider} in the chain to delegate requests to.
     * @param next    A function returning the next {@link InputProvider} in the chain, given the data requested by the
     *                current one.
     */
    public ChainedInputProvider(final InputProvider current,
            final FunctionE<Map<String, Object>, InputProvider, InputProviderException> next) {
        this.current = current;
        this.next = next;
    }

    @Override
    @SafeVarargs
    public final ChainedInputProvider needString(final String id, final String prompt,
            final Optional<String> defaultValue,
            final Validator<String>... validators) throws InputProviderException {

        this.current.needString(id, prompt, defaultValue, validators);
        return this;
    }

    @Override
    public ChainedInputProvider fixedString(final String id, final String fixedValue) throws InputProviderException {
        this.current.fixedString(id, fixedValue);
        return this;
    }

    @Override
    @SafeVarargs
    public final ChainedInputProvider needInteger(final String id, final String prompt,
            final Optional<Integer> defaultValue,
            final Validator<Integer>... validators) throws InputProviderException {

        this.current.needInteger(id, prompt, defaultValue, validators);
        return this;
    }

    @Override
    public ChainedInputProvider fixedInteger(final String id, final Integer fixedValue) throws InputProviderException {
        this.current.fixedInteger(id, fixedValue);
        return this;
    }

    @Override
    @SafeVarargs
    public final ChainedInputProvider needBoolean(final String id, final String prompt,
            final Optional<Boolean> defaultValue,
            final Validator<Boolean>... validators) throws InputProviderException {

        this.current.needBoolean(id, prompt, defaultValue, validators);
        return this;
    }

    @Override
    public ChainedInputProvider fixedBoolean(final String id, final Boolean fixedValue) throws InputProviderException {
        this.current.fixedBoolean(id, fixedValue);
        return this;
    }

    @Override
    public ChainedInputProvider needObject(final String id, final String prompt, final Optional<Object> defaultValue,
            final Set<?> objectSet) throws InputProviderException {

        this.current.needObject(id, prompt, defaultValue, objectSet);
        return this;
    }

    @Override
    public ChainedInputProvider fixedObject(final String id, final Object fixedValue) throws InputProviderException {
        this.current.fixedObject(id, fixedValue);
        return this;
    }

    @Override
    public Map<String, Object> requestData(final String title) throws InputProviderException {
        return this.current.requestData(title);
    }

    @Override
    public InputProvider getNext(final Map<String, Object> lastDataSet) throws InputProviderException {
        return this.next.apply(lastDataSet);
    }
}
